#!/bin/bash

# computes (an approximation of) the primal integral from a solvetrace file
# the primal integral is defined in
#   Tobias Achterberg, Timo Berthold, Gregor Hendel
#   Rounding and Propagation Heuristics for Mixed Integer Programming
#   Operations Research Proceedings 2011, 07.2012.
#   http://nbn-resolving.de/urn:nbn:de:0297-zib-13255
# and can be used to evaluate the ability of a solver to find good primal
# solutions early in the search
#
# usage: ./getprimint.sh <solvetracefile>
#
# assumes minimization problems
# the accuracy of the computed value depends on the resolution of the solvetrace file

#opt=`grep $2 benchmark.solu | awk '{print $3}'`

# optimal value for sp98ir
opt=219676790.4

# fields are lineNum, seriesID, node, seconds, bestFound, bestBound

awk -F, -vopt=$opt '
function abs(x) { return x >= 0.0 ? x : -x }
function max(x,y) { return x > y ? x : y }
BEGIN { gamma = 1.0; }
/^*/ { next; }
//   {
  time=$4
  pb=$5

  primint += gamma * (time - lasttime);

  if( pb ~ "na" || (pb * opt) < 0.0 )
    gamma = 1.0;
  else if( opt == 0.0 && pb == 0.0 )
    gamma = 0.0;
  else
    gamma = abs(opt - pb) / max(abs(opt), abs(pb));

  lasttime = time;

  #printf("%8.2fs: %.6e %.6e -> %.3e  %g\n", time, opt, pb, gamma, primint);
}
END { print primint; }
' $1

